@@ -2,7 +2,7 @@
  * vlc_es_out.h: es_out (demuxer output) descriptor, queries and methods
  *****************************************************************************
  * Copyright (C) 1999-2004 the VideoLAN team
- * $Id: bd654484a972d30c5b23de24c00dead6a9cd90fe $
+ * $Id: 043e5579d86e61b3c9be1eae090cd894f4b0cc0c $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -44,20 +44,12 @@ enum es_out_mode_e
 
 enum es_out_query_e
 {
-    /* activate application of mode */
-    ES_OUT_SET_ACTIVE,  /* arg1= bool                     */
-    /* see if mode is currently aplied or not */
-    ES_OUT_GET_ACTIVE,  /* arg1= bool*                    */
-
-    /* set/get mode */
-    ES_OUT_SET_MODE,    /* arg1= int                            */
-    ES_OUT_GET_MODE,    /* arg2= int*                           */
-
     /* set ES selected for the es category (audio/video/spu) */
     ES_OUT_SET_ES,      /* arg1= es_out_id_t*                   */
+    ES_OUT_RESTART_ES,  /* arg1= es_out_id_t*                   */
 
     /* set 'default' tag on ES (copied across from container) */
-    ES_OUT_SET_DEFAULT, /* arg1= es_out_id_t*                   */
+    ES_OUT_SET_ES_DEFAULT, /* arg1= es_out_id_t*                */
 
     /* force selection/unselection of the ES (bypass current mode) */
     ES_OUT_SET_ES_STATE,/* arg1= es_out_id_t* arg2=bool   */
@@ -65,48 +57,58 @@ enum es_out_query_e
 
     /* */
     ES_OUT_SET_GROUP,   /* arg1= int                            */
-    ES_OUT_GET_GROUP,   /* arg1= int*                           */
 
     /* PCR handling, DTS/PTS will be automatically computed using thoses PCR
      * XXX: SET_PCR(_GROUP) are in charge of the pace control. They will wait
      * to slow down the demuxer so that it reads at the right speed.
-     * XXX: if you want PREROLL just call RESET_PCR and
-     * ES_OUT_SET_NEXT_DISPLAY_TIME and send data to the decoder *without*
-     * calling SET_PCR until preroll is finished.
+     * XXX: if you want PREROLL just call ES_OUT_SET_NEXT_DISPLAY_TIME and send
+     * as you would normally do.
      */
     ES_OUT_SET_PCR,             /* arg1=int64_t i_pcr(microsecond!) (using default group 0)*/
     ES_OUT_SET_GROUP_PCR,       /* arg1= int i_group, arg2=int64_t i_pcr(microsecond!)*/
     ES_OUT_RESET_PCR,           /* no arg */
 
-    /* Timestamp handling, convert an input timestamp to a global clock one.
-     * (shouldn't have to be used by input plugins directly) */
-    ES_OUT_GET_TS,             /* arg1=int64_t i_ts(microsecond!) (using default group 0), arg2=int64_t* converted i_ts */
-
     /* Try not to use this one as it is a bit hacky */
-    ES_OUT_SET_FMT,     /* arg1= es_out_id_t* arg2=es_format_t* */
+    ES_OUT_SET_ES_FMT,         /* arg1= es_out_id_t* arg2=es_format_t* */
 
-    /* Allow preroll of data (data with dts/pts < i_pts for one ES will be decoded but not displayed */
-    ES_OUT_SET_NEXT_DISPLAY_TIME,   /* arg1=es_out_id_t* arg2=int64_t i_pts(microsecond) */
+    /* Allow preroll of data (data with dts/pts < i_pts for all ES will be decoded but not displayed */
+    ES_OUT_SET_NEXT_DISPLAY_TIME,       /* arg1=int64_t i_pts(microsecond) */
     /* Set meta data for group (dynamic) */
     ES_OUT_SET_GROUP_META,  /* arg1=int i_group arg2=vlc_meta_t */
     /* Set epg for group (dynamic) */
     ES_OUT_SET_GROUP_EPG,   /* arg1=int i_group arg2=vlc_epg_t */
     /* */
-    ES_OUT_DEL_GROUP        /* arg1=int i_group */
+    ES_OUT_DEL_GROUP,       /* arg1=int i_group */
+
+    /* Set scrambled state for one es */
+    ES_OUT_SET_ES_SCRAMBLED_STATE,  /* arg1=int i_group arg2=es_out_id_t* */
+
+    /* Stop any buffering being done, and ask if es_out has no more data to
+     * play.
+     * It will not block and so MUST be used carrefully. The only good reason
+     * is for interactive playback (like for DVD menu).
+     * XXX You SHALL call ES_OUT_RESET_PCR before any other es_out_Control/Send calls. */
+    ES_OUT_GET_EMPTY,       /* arg1=bool*   res=cannot fail */
+
+    /* First value usable for private control */
+    ES_OUT_PRIVATE_START = 0x10000,
 };
 
 struct es_out_t
 {
-    es_out_id_t *(*pf_add)    ( es_out_t *, es_format_t * );
+    es_out_id_t *(*pf_add)    ( es_out_t *, const es_format_t * );
     int          (*pf_send)   ( es_out_t *, es_out_id_t *, block_t * );
     void         (*pf_del)    ( es_out_t *, es_out_id_t * );
     int          (*pf_control)( es_out_t *, int i_query, va_list );
-    bool      b_sout;
+    void         (*pf_destroy)( es_out_t * );
+
+    bool         b_sout;
 
     es_out_sys_t    *p_sys;
 };
 
-static inline es_out_id_t * es_out_Add( es_out_t *out, es_format_t *fmt )
+LIBVLC_USED
+static inline es_out_id_t * es_out_Add( es_out_t *out, const es_format_t *fmt )
 {
     return out->pf_add( out, fmt );
 }
@@ -138,6 +140,11 @@ static inline int es_out_Control( es_out_t *out, int i_query, ... )
     return i_result;
 }
 
+static inline void es_out_Delete( es_out_t *p_out )
+{
+    p_out->pf_destroy( p_out );
+}
+
 /**
  * @}
  */
