@@ -2,7 +2,7 @@
  * callback.h : Callbacks for CD digital audio input module
  *****************************************************************************
  * Copyright (C) 2004 the VideoLAN team
- * $Id: callback.h 8606 2004-08-31 18:32:54Z rocky $
+ * $Id: dc290141e21cd054ae5c1cce7f6a03525620cbab $
  *
  * Authors: Rocky Bernstein <rocky@panix.com>
  *
@@ -18,12 +18,16 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
-#include <vlc/vlc.h>
+#ifdef HAVE_CONFIG_H
+#   include "config.h"
+#endif
 
-/* 
+#include <vlc_common.h>
+
+/*
    Minimum, maximum and default number of blocks we allow on read.
 */
 #define MIN_BLOCKS_PER_READ 1
@@ -40,19 +44,19 @@ int  CDDBEnabledCB( vlc_object_t *p_this, const char *psz_name,
 
 
 int  CDTextEnabledCB( vlc_object_t *p_this, const char *psz_name,
-			  vlc_value_t oldval, vlc_value_t val,
-			  void *p_data );
+              vlc_value_t oldval, vlc_value_t val,
+              void *p_data );
 
 int  CDTextPreferCB( vlc_object_t *p_this, const char *psz_name,
-			 vlc_value_t oldval, vlc_value_t val,
-			 void *p_data );
+             vlc_value_t oldval, vlc_value_t val,
+             void *p_data );
 
 int  CDDANavModeCB( vlc_object_t *p_this, const char *psz_name,
-		    vlc_value_t oldval, vlc_value_t val,
-		    void *p_data );
+            vlc_value_t oldval, vlc_value_t val,
+            void *p_data );
 
 
 int CDDABlocksPerReadCB ( vlc_object_t *p_this, const char *psz_name,
-			      vlc_value_t oldval, vlc_value_t val, 
-			      void *p_data );
+                  vlc_value_t oldval, vlc_value_t val,
+                  void *p_data );
 
