@@ -2,7 +2,7 @@
  * equalizer.m: MacOS X interface module
  *****************************************************************************
  * Copyright (C) 2004-2008 the VideoLAN team
- * $Id: 78adadd89f881f6ebf56e8b2f5a47a3ab889a5f7 $
+ * $Id: 0c726b1a868e54b94edc6ca7a5768227a2e62ba9 $
  *
  * Authors: Jérôme Decoodt <djc@videolan.org>
  *          Felix Paul Kühne <fkuehne -at- videolan -dot- org>
@@ -54,7 +54,7 @@ static void ChangeFiltersString( intf_thread_t *p_intf,
     aout_instance_t *p_aout = (aout_instance_t *)p_object;
     if( !p_object )
     {
-        p_object = (vlc_object_t *)pl_Yield( p_intf );
+        p_object = (vlc_object_t *)pl_Hold( p_intf );
     }
 
     psz_string = var_GetNonEmptyString( p_object, "audio-filter" );
@@ -112,9 +112,6 @@ static void ChangeFiltersString( intf_thread_t *p_intf,
     {
         /* save changed to config */
         config_PutPsz( p_object, "audio-filter", psz_string );
-
-        /* save to vlcrc */
-        config_SaveConfigFile( p_object, "main" );
     }
     
     free( psz_string );
@@ -128,7 +125,7 @@ static bool GetFiltersStatus( intf_thread_t *p_intf,
     vlc_object_t *p_object = vlc_object_find( p_intf,
                                 VLC_OBJECT_AOUT, FIND_ANYWHERE );
     if( p_object == NULL )
-        p_object = (vlc_object_t *)pl_Yield( p_intf );
+        p_object = (vlc_object_t *)pl_Hold( p_intf );
 
     if( (BOOL)config_GetInt( p_intf, "macosx-eq-keep" ) == YES )
         psz_string = config_GetPsz( p_intf, "audio-filter" );
@@ -184,7 +181,7 @@ static bool GetFiltersStatus( intf_thread_t *p_intf,
                                               VLC_OBJECT_AOUT, FIND_ANYWHERE );
 
     if( p_object == NULL )
-        p_object = (vlc_object_t *)pl_Yield( p_intf );
+        p_object = (vlc_object_t *)pl_Hold( p_intf );
 
     var_Create( p_object, "equalizer-preamp", VLC_VAR_FLOAT |
                 VLC_VAR_DOINHERIT );
@@ -218,11 +215,7 @@ static bool GetFiltersStatus( intf_thread_t *p_intf,
     for( i = 0; i < 10; i++ )
     {
         /* Read dB -20/20 */
-#ifdef HAVE_STRTOF
         f_band[i] = strtof( psz_bands, &p_next );
-#else
-        f_band[i] = (float)strtod( psz_bands, &p_next );
-#endif
         if( !p_next || p_next == psz_bands ) break; /* strtof() failed */
     
         if( !*psz_bands ) break; /* end of line */
@@ -244,25 +237,21 @@ static bool GetFiltersStatus( intf_thread_t *p_intf,
                                               VLC_OBJECT_AOUT, FIND_ANYWHERE );
 
     if( p_object == NULL )
-        p_object = (vlc_object_t *)pl_Yield( p_intf );
+        p_object = (vlc_object_t *)pl_Hold( p_intf );
 
-    char psz_values[102];
-    memset( psz_values, 0, 102 );
-
-    /* Write the new bands values */
-    /* TODO: write a generic code instead of ten times the same thing */
-
-    sprintf( psz_values, "%s %.1f", psz_values, [o_slider_band1 floatValue] );
-    sprintf( psz_values, "%s %.1f", psz_values, [o_slider_band2 floatValue] );
-    sprintf( psz_values, "%s %.1f", psz_values, [o_slider_band3 floatValue] );
-    sprintf( psz_values, "%s %.1f", psz_values, [o_slider_band4 floatValue] );
-    sprintf( psz_values, "%s %.1f", psz_values, [o_slider_band5 floatValue] );
-    sprintf( psz_values, "%s %.1f", psz_values, [o_slider_band6 floatValue] );
-    sprintf( psz_values, "%s %.1f", psz_values, [o_slider_band7 floatValue] );
-    sprintf( psz_values, "%s %.1f", psz_values, [o_slider_band8 floatValue] );
-    sprintf( psz_values, "%s %.1f", psz_values, [o_slider_band9 floatValue] );
-    sprintf( psz_values, "%s %.1f", psz_values, [o_slider_band10 floatValue] );
+    const char *psz_values;
+    NSString *preset = [NSString stringWithFormat:@"%.1f", [o_slider_band1 floatValue] ];
+    preset = [preset stringByAppendingFormat:@"%.1f ", [o_slider_band2 floatValue] ];
+    preset = [preset stringByAppendingFormat:@"%.1f ", [o_slider_band3 floatValue] ];
+    preset = [preset stringByAppendingFormat:@"%.1f ", [o_slider_band4 floatValue] ];
+    preset = [preset stringByAppendingFormat:@"%.1f ", [o_slider_band5 floatValue] ];
+    preset = [preset stringByAppendingFormat:@"%.1f ", [o_slider_band6 floatValue] ];
+    preset = [preset stringByAppendingFormat:@"%.1f ", [o_slider_band7 floatValue] ];
+    preset = [preset stringByAppendingFormat:@"%.1f ", [o_slider_band8 floatValue] ];
+    preset = [preset stringByAppendingFormat:@"%.1f ", [o_slider_band9 floatValue] ];
+    preset = [preset stringByAppendingFormat:@"%.1f", [o_slider_band10 floatValue] ];
 
+    psz_values = [preset UTF8String];
     var_SetString( p_object, "equalizer-bands", psz_values );
 
     if( (BOOL)config_GetInt( p_intf, "macosx-eq-keep" ) == YES )
@@ -284,15 +273,17 @@ static bool GetFiltersStatus( intf_thread_t *p_intf,
     vlc_object_t *p_object= vlc_object_find( p_intf,
                                              VLC_OBJECT_AOUT, FIND_ANYWHERE );
     if( p_object == NULL )
-        p_object = (vlc_object_t *)pl_Yield( p_intf );
-
-    char psz_values[102];
-    memset( psz_values, 0, 102 );
+        p_object = (vlc_object_t *)pl_Hold( p_intf );
 
     var_SetString( p_object , "equalizer-preset" , preset_list[[sender indexOfSelectedItem]] );
 
+    NSString *preset = @"";
+    const char *psz_values;
     for( i = 0; i < 10; i++ )
-        sprintf( psz_values, "%s %.1f", psz_values, eqz_preset_10b[[sender indexOfSelectedItem]]->f_amp[i] );
+    {
+        preset = [preset stringByAppendingFormat:@"%.1f ", eqz_preset_10b[[sender indexOfSelectedItem]]->f_amp[i] ];
+    }
+    psz_values = [preset UTF8String];
     var_SetString( p_object, "equalizer-bands", psz_values );
     var_SetFloat( p_object, "equalizer-preamp", eqz_preset_10b[[sender indexOfSelectedItem]]->f_preamp);
 
@@ -327,7 +318,7 @@ static bool GetFiltersStatus( intf_thread_t *p_intf,
     vlc_object_t *p_object = vlc_object_find( p_intf,
                                               VLC_OBJECT_AOUT, FIND_ANYWHERE );
     if( p_object == NULL )
-        p_object = (vlc_object_t *)pl_Yield( p_intf );
+        p_object = (vlc_object_t *)pl_Hold( p_intf );
 
     var_SetFloat( p_object, "equalizer-preamp", f_preamp );
 
@@ -365,7 +356,7 @@ static bool GetFiltersStatus( intf_thread_t *p_intf,
                                              VLC_OBJECT_AOUT, FIND_ANYWHERE );
     aout_instance_t *p_aout = (aout_instance_t *)p_object;
     if( p_object == NULL )
-        p_object = (vlc_object_t *)pl_Yield( p_intf );
+        p_object = (vlc_object_t *)pl_Hold( p_intf );
 
     var_SetBool( p_object, "equalizer-2pass", b_2p );
     if( ( [o_ckb_enable state] ) && ( p_aout != NULL ) )
@@ -400,7 +391,7 @@ static bool GetFiltersStatus( intf_thread_t *p_intf,
     vlc_object_t *p_object= vlc_object_find( VLCIntf,
                                              VLC_OBJECT_AOUT, FIND_ANYWHERE );
     if( p_object == NULL )
-        p_object = (vlc_object_t *)pl_Yield( VLCIntf );
+        p_object = (vlc_object_t *)pl_Hold( VLCIntf );
 
     [o_window setExcludedFromWindowsMenu: TRUE];
 
@@ -460,7 +451,7 @@ static bool GetFiltersStatus( intf_thread_t *p_intf,
 }
 
 
-- (id)getSliderByIndex:(int)index
+- (id)sliderByIndex:(int)index
 {
     switch(index)
     {
@@ -498,7 +489,7 @@ static bool GetFiltersStatus( intf_thread_t *p_intf,
 
 - (void)setValue:(float)value forSlider:(int)index
 {
-    id slider = [self getSliderByIndex:index];
+    id slider = [self sliderByIndex:index];
 
     if (slider != nil)
     {
