@@ -2,9 +2,9 @@
  * callbacks.h : pda plugin for vlc
  *****************************************************************************
  * Copyright (C) 2003 the VideoLAN team
- * $Id: pda_callbacks.h 11664 2005-07-09 06:17:09Z courmisch $
+ * $Id: d8563beb755bfc1c52c434393c2bacd58d6a7334 $
  *
- * Authors: Jean-Paul Saman <jpsaman@wxs.nl>
+ * Authors: Jean-Paul Saman <jpsaman _at_ videolan _dot_ org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,16 +18,16 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #include <gtk/gtk.h>
 
-#include <vlc/vlc.h>
-#include <vlc/intf.h>
+#include <vlc_common.h>
+#include <vlc_interface.h>
 
 void ReadDirectory( intf_thread_t *p_intf, GtkListStore *p_list, char *psz_dir );
-void PlaylistRebuildListStore( GtkListStore *p_list, playlist_t * p_playlist );
+void PlaylistRebuildListStore( intf_thread_t *p_intf, GtkListStore *p_list, playlist_t * p_playlist );
 
 
 gboolean
