@@ -1,11 +1,11 @@
 /*****************************************************************************
  * cmd_generic.hpp
  *****************************************************************************
- * Copyright (C) 2003 VideoLAN
- * $Id: cmd_generic.hpp 9641 2004-12-22 13:06:43Z gbazin $
+ * Copyright (C) 2003 the VideoLAN team
+ * $Id: f53995e8c1cad614a7984b4f54d3f577c8140b25 $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
- *          Olivier Teulire <ipkiss@via.ecp.fr>
+ *          Olivier Teulière <ipkiss@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #ifndef CMD_GENERIC_HPP
@@ -44,6 +44,23 @@ class Cmd##name: public CmdGeneric \
 };
 
 
+/// Macro to define a "callback" command inside a class
+#define DEFINE_CALLBACK( parent, action ) \
+class Cmd##action: public CmdGeneric \
+{ \
+    public: \
+        Cmd##action( parent *pParent ): \
+            CmdGeneric( pParent->getIntf() ), m_pParent( pParent ) {} \
+        virtual ~Cmd##action() {} \
+        virtual void execute(); \
+        virtual string getType() const { return "Cmd" #parent #action; } \
+    private: \
+        parent *m_pParent; \
+\
+} m_cmd##action; \
+friend class Cmd##action;
+
+
 /// Base class for skins commands
 class CmdGeneric: public SkinObject
 {
@@ -56,6 +73,10 @@ class CmdGeneric: public SkinObject
         /// Return the type of the command
         virtual string getType() const { return ""; }
 
+        /// During queue reductions, check if we really want to remove
+        /// this command.
+        virtual bool checkRemove( CmdGeneric * ) const { return true; }
+
     protected:
         CmdGeneric( intf_thread_t *pIntf ): SkinObject( pIntf ) {}
 };
