@@ -1,8 +1,8 @@
 /*****************************************************************************
  * macosx_window.hpp
  *****************************************************************************
- * Copyright (C) 2003 VideoLAN
- * $Id: macosx_window.hpp 9711 2005-01-09 19:46:39Z asmax $
+ * Copyright (C) 2003 the VideoLAN team
+ * $Id: 8c58d447510a59186c2c3e6e454fb432b4c670b7 $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *
@@ -18,13 +18,14 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #ifndef MACOSX_WINDOW_HPP
 #define MACOSX_WINDOW_HPP
 
 #include "../src/os_window.hpp"
+#include <Carbon/Carbon.h>
 
 class MacOSXDisplay;
 class MacOSXDragDrop;
@@ -59,11 +60,16 @@ class MacOSXWindow: public OSWindow
         /// Toggle the window on top
         virtual void toggleOnTop( bool onTop ) const;
 
+        /// Get the Carbon window handle
+        WindowRef getWindowRef() const { return m_win; };
+
     private:
         /// Parent window
         MacOSXWindow *m_pParent;
         /// Indicates whether the window handles drag&drop events
         bool m_dragDrop;
+        /// Carbon Window object
+        WindowRef m_win;
 };
 
 
