@@ -1,11 +1,11 @@
 /*****************************************************************************
  * win32_window.cpp
  *****************************************************************************
- * Copyright (C) 2003 VideoLAN
- * $Id: win32_window.cpp 10101 2005-03-02 16:47:31Z robux4 $
+ * Copyright (C) 2003 the VideoLAN team
+ * $Id: 4725d83592df6cf9b3fdee0984e588529fb255d9 $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
- *          Olivier Teulire <ipkiss@via.ecp.fr>
+ *          Olivier Teulière <ipkiss@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #ifdef WIN32_SKINS
@@ -38,28 +38,36 @@
 #endif
 
 
+// XXX layered windows are supposed to work only with at least win2k
+#ifndef WS_EX_LAYERED
+#   define WS_EX_LAYERED 0x00080000
+#endif
+
 Win32Window::Win32Window( intf_thread_t *pIntf, GenericWindow &rWindow,
                           HINSTANCE hInst, HWND hParentWindow,
                           bool dragDrop, bool playOnDrop,
                           Win32Window *pParentWindow ):
-    OSWindow( pIntf ), m_dragDrop( dragDrop ), m_isLayered( false )
+    OSWindow( pIntf ), m_dragDrop( dragDrop ), m_isLayered( false ),
+    m_pParent( pParentWindow )
 {
     // Create the window
     if( pParentWindow )
     {
         // Child window (for vout)
-        HWND hParent = pParentWindow->getHandle();
-        m_hWnd = CreateWindowEx( WS_EX_TOOLWINDOW, "SkinWindowClass",
-            "default name", WS_CHILD, CW_USEDEFAULT, CW_USEDEFAULT,
-            CW_USEDEFAULT, CW_USEDEFAULT, hParent, 0, hInst, NULL );
+        m_hWnd_parent = pParentWindow->getHandle();
+        m_hWnd = CreateWindowEx( WS_EX_TOOLWINDOW | WS_EX_NOPARENTNOTIFY,
+                     "SkinWindowClass", "default name", WS_CHILD,
+                     CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
+                     m_hWnd_parent, 0, hInst, NULL );
     }
     else
     {
         // Normal window
+        m_hWnd_parent = hParentWindow;
         m_hWnd = CreateWindowEx( WS_EX_TOOLWINDOW, "SkinWindowClass",
             "default name", WS_POPUP | WS_CLIPCHILDREN,
             CW_USEDEFAULT, CW_USEDEFAULT,
-            CW_USEDEFAULT, CW_USEDEFAULT, hParentWindow, 0, hInst, NULL );
+            CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd_parent, 0, hInst, NULL );
     }
 
     if( !m_hWnd )
@@ -80,12 +88,6 @@ Win32Window::Win32Window( intf_thread_t *pIntf, GenericWindow &rWindow,
         // Register the window as a drop target
         RegisterDragDrop( m_hWnd, m_pDropTarget );
     }
-
-    // XXX Set this window as the vout
-    if( pParentWindow )
-    {
-        VlcProc::instance( getIntf() )->setVoutWindow( (void*)m_hWnd );
-    }
 }
 
 
@@ -108,6 +110,14 @@ Win32Window::~Win32Window()
 }
 
 
+void Win32Window::reparent( void* OSHandle, int x, int y, int w, int h )
+{
+    // Reparent the window
+    SetParent( m_hWnd, (HWND)OSHandle );
+    MoveWindow( m_hWnd, x, y, w, h, true );
+}
+
+
 void Win32Window::show( int left, int top ) const
 {
     ShowWindow( m_hWnd, SW_SHOW );
