@@ -2,10 +2,10 @@
  * x11_dragdrop.cpp
  *****************************************************************************
  * Copyright (C) 2003 the VideoLAN team
- * $Id: x11_dragdrop.cpp 11664 2005-07-09 06:17:09Z courmisch $
+ * $Id: fd77b6c103187891b4a5e3c261b33341649ca7e7 $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
- *          Olivier Teulire <ipkiss@via.ecp.fr>
+ *          Olivier Teulière <ipkiss@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #ifdef X11_SKINS
@@ -186,10 +186,9 @@ void X11DragDrop::dndDrop( ldata_t data )
         selection = selection.substr( 0, end - 1 );
 
         // Find the protocol, if any
-        string::size_type pos = selection.find( ":", 0 );
-        if( selection.find( "///", pos + 1 ) == pos + 1 )
+        if( selection.find( "file://", 0 ) == 0 )
         {
-            selection.erase( pos + 1, 2 );
+            selection.erase( 0, 7 );
         }
 
         char *psz_fileName = new char[selection.size() + 1];
