@@ -3,7 +3,7 @@
  *****************************************************************************
  * Copyright (C) 2005 the VideoLAN team
  *
- * $Id: 9de23606c067a678eb12a250120f4408176ed46c $
+ * $Id: 7a45d688a28d65d470d5cd4c2565626485c86ac8 $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Filippo Carone <littlejohn@videolan.org>
@@ -476,7 +476,7 @@ void libvlc_video_set_spu( libvlc_media_player_t *p_mi, int i_spu,
         goto end;
     }
 
-    if( (i_spu < 0) && (i_spu > val_list.p_list->i_count) )
+    if( (i_spu < 0) || (i_spu > val_list.p_list->i_count) )
     {
         libvlc_exception_raise( p_e, "Subtitle value out of range" );
         goto end;
