@@ -2,7 +2,7 @@
  * update.c: VLC update checking and downloading
  *****************************************************************************
  * Copyright © 2005-2008 the VideoLAN team
- * $Id: f495b74dd1a2a22dde4ecd01da7daba7a1b79466 $
+ * $Id: a0f43346363549d4faf3fbf643d5e0d69597b5e0 $
  *
  * Authors: Antoine Cellerier <dionoea -at- videolan -dot- org>
  *          Rémi Duraffort <ivoire at via.ecp.fr>
@@ -35,9 +35,6 @@
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
-#ifdef HAVE_SYS_STAT_H
-#   include <sys/stat.h>
-#endif
 
 #include <vlc_common.h>
 #include <vlc_update.h>
@@ -1495,12 +1492,10 @@ static void* update_DownloadReal( vlc_object_t *p_this );
  * Download the file given in the update_t
  *
  * \param p_update structure
- * \param destination to store the download file
- *        This can be an existing dir, a (non)existing target fullpath filename or
- *        NULL for the current working dir.
+ * \param dir to store the download file
  * \return nothing
  */
-void update_Download( update_t *p_update, const char *destination )
+void update_Download( update_t *p_update, const char *psz_destdir )
 {
     assert( p_update );
 
@@ -1520,7 +1515,7 @@ void update_Download( update_t *p_update, const char *destination )
 
     p_udt->p_update = p_update;
     p_update->p_download = p_udt;
-    p_udt->psz_destination = destination ? strdup( destination ) : NULL;
+    p_udt->psz_destdir = psz_destdir ? strdup( psz_destdir ) : NULL;
 
     vlc_thread_create( p_udt, "download update", update_DownloadReal,
                        VLC_THREAD_PRIORITY_LOW );
@@ -1540,14 +1535,13 @@ static void* update_DownloadReal( vlc_object_t *p_this )
     char *psz_tmpdestfile = NULL;
 
     FILE *p_file = NULL;
-    struct stat p_stat;
     stream_t *p_stream = NULL;
     void* p_buffer = NULL;
     int i_read;
     int canc;
 
     update_t *p_update = p_udt->p_update;
-    char *psz_destination = p_udt->psz_destination;
+    char *psz_destdir = p_udt->psz_destdir;
 
     msg_Dbg( p_udt, "Opening Stream '%s'", p_update->release.psz_url );
     canc = vlc_savecancel ();
@@ -1573,15 +1567,8 @@ static void* update_DownloadReal( vlc_object_t *p_this )
     }
     psz_tmpdestfile++;
 
-    if( utf8_stat( psz_destination, &p_stat) == 0 && (p_stat.st_mode & S_IFDIR) )
-    {
-        if( asprintf( &psz_destfile, "%s%c%s", psz_destination, DIR_SEP_CHAR, psz_tmpdestfile ) == -1 )
-            goto end;
-    }
-    else if( psz_destination )
-        psz_destfile = strdup( psz_destination );
-    else
-        psz_destfile = strdup( psz_tmpdestfile );
+    if( asprintf( &psz_destfile, "%s%s", psz_destdir, psz_tmpdestfile ) == -1 )
+        goto end;
 
     p_file = utf8_fopen( psz_destfile, "w" );
     if( !p_file )
@@ -1742,11 +1729,12 @@ end:
         stream_Delete( p_stream );
     if( p_file )
         fclose( p_file );
+
+    free( psz_destdir );
     free( psz_destfile );
     free( p_buffer );
     free( psz_size );
 
-    free( p_udt->psz_destination );
     p_udt->p_update->p_download = NULL;
 
     vlc_object_release( p_udt );
